/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 *  WebSphere sMash
 *
 * (C) Copyright IBM Corp. 2008  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */

/*-------------------------------------------------------------------
*  access via URL like:
*    http://localhost:8070/resources/applications/{applicationsId}/threads/{threadId}/stackFrames/{stackeFrameId}/localVariables
-------------------------------------------------------------------*/
import zero.ide.services.DebugUtils
import com.sun.jdi.ArrayReference
import com.sun.jdi.ObjectReference
import com.sun.jdi.ReferenceType
import com.sun.jdi.ClassNotLoadedException
import com.sun.jdi.InvalidStackFrameException
import com.sun.jdi.AbsentInformationException
import zero.appbuilder.framework.Framework

/*
*
* @success 200 Returns list of local variables for the stack frame
* @format application/json or text/html depending on Accept header
*
*/
def onList() {
    def appId = request.params.applicationsId[]
    def frameId = request.params.stackFramesId[]
    def threadId = request.params.threadsId[]
    threadId = new Integer(threadId)
    def frameIndex = request.params.stackFramesId[]
    frameIndex = new Integer(frameIndex)
    
    def vars
    if (0 == threadId) {
        // PHP
        def phpThreadService = Framework.createServiceContext().getService('phpthreads')
        if (phpThreadService) {
            // TODO
            def thread = phpThreadService.getThread(appId, threadId)
            framesList = thread.getStackFrames()
            def frame = framesList[frameIndex]
            vars = frame.getVariables()
        }
    } else {
        // Java/Groovy
        def debugger = DebugUtils.getDebugger();
        if(null==debugger) {
            logger.FINE {DebugUtils.DEBUG_NULL_ERR}
            request.status = 404
            return
        }
        def debugVM = debugger.getVM()
        if(null==debugVM) {
            logger.FINE {DebugUtils.VM_NULL_ERR}
            request.status = 404
            return
        }
        def thread = DebugUtils.getThreadReference()
        if(thread==null) {
            logger.FINE {DebugUtils.THREAD_NULL_ERR}
            request.status = 404
            return
        }
        
        try {
            vars = DebugUtils.getLocalVariables(thread, Integer.parseInt(frameId))
        } catch(AbsentInformationException e) {
            def message = "Exception while trying to read variables from target VM: " + e.getMessage()
            logger.FINE {message}
            request.status = 404
            return
        }
    }
    
    returnJsonObject(vars)
}

def returnJsonObject(object) {
	request.json.output = object
    request.view = "JSON"
    render();
    request.status = 200
}

/**
*
* @success 200 Returns name, type, toString() value, and uris of fields
* @error 404 variable not found 
* @format application/json or text/html depending on Accept header
*
*/
def onRetrieve() {
    def appId = request.params.applicationsId[]
    def frameId = request.params.stackFramesId[]
    def threadId = request.params.threadsId[]
    def localVariableId = request.params.localVariablesId[]
    
    def var
    if (0 == threadId) {
        // PHP
        def phpThreadService = Framework.createServiceContext().getService('phpthreads')
        if (phpThreadService) {
            // TODO
            def thread = phpThreadService.getThread(appId, threadId)
            framesList = thread.getStackFrames()
            def frame = framesList[frameIndex]
            vars = frame.getVariables()
            var = vars[localVariableId]
        }
    } else {
        // Java/Groovy
        def debugger = DebugUtils.getDebugger()
        if(null==debugger) {
            logger.FINE {DebugUtils.DEBUG_NULL_ERR}
            request.status = 404
            return
        }
        def debugVM = debugger.getVM()
        if(null==debugVM) {
            logger.FINE {DebugUtils.VM_NULL_ERR}
            request.status = 404
            return
        }
        
        try {
            var = DebugUtils.getLocalVariable(DebugUtils.getThreadReference(), Integer.parseInt(frameId), request.path[])       
        } catch(AbsentInformationException e) {
            logger.FINE {e.getMessage()}
            request.status = 404
            return
        }
    }

    returnJsonObject(var)
}
