/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 *  WebSphere sMash
 *
 * (C) Copyright IBM Corp. 2008  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */

import zero.appbuilder.applications.ApplicationRegistry
import zero.ide.services.GroovySourceInfo
import zero.core.utils.ClientMessageFormatter

def onRetrieve() {
	def uri = request.path[]
	def uriParts = uri.split("/")
	def appId = uriParts[3]
	def srcPath = uri.substring(getSourcePathStart(uri, appId), uri.lastIndexOf('/'))
	def apphome = ApplicationRegistry.get(appId).getAppHome()
	def key = uriParts[uriParts.length-1]	
	
	if (srcPath.endsWith(".groovy")) {
		String filename="";
		try {
			def file = new File(apphome,srcPath)
			if(!file.exists()) {
				request.status = 404
				return
			}
			filename=file.getCanonicalPath();
			def srcInfo = new GroovySourceInfo(apphome,file)
			if(key=="executables") {			
				returnJsonObject(srcInfo.getExecutables())
			} else if(key=="methods") {
				returnJsonObject(srcInfo.getMethods())
			} else {
				// looking for a specific line
				def line = Integer.parseInt(key)
				def details = [
				               isExecutable: srcInfo.isExecutable(line),
				               isMethod: srcInfo.isMethodSignature(line)
				               ]
				returnJsonObject(details)
			}
		}
		catch (Exception e) {
	        ClientMessageFormatter messageFormatter = new ClientMessageFormatter("zero/appbuilder/core/resources/consolemessages");
	        request.status = 500
			request.headers.out.'Content-Type' = 'application/json'
    		request.view = 'error'
            request.error.message = messageFormatter.getMessage('FILE_DOES_NOT_COMPILE') + "\r\n" + e.getMessage();
    		render()
		}
	} else {
		request.status = 404
	}	
	
}

def getSourcePathStart(uri, appId) {
	def prefix = "/resources/applications/"+appId+"/sourceDetails/"
	return prefix.length()
}

def returnJsonObject(object) {
	request.json.output = object
    request.view = "JSON"
    render();
    request.status = 200
}
