/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 *  WebSphere sMash
 *
 * (C) Copyright IBM Corp. 2008  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */

import zero.ide.services.DebugUtils
import com.sun.jdi.InvalidStackFrameException
import com.sun.jdi.AbsentInformationException
import zero.appbuilder.framework.Framework
/*-------------------------------------------------------------------
 access via URL like:
    http://localhost:8070/resources/applications/{appId}/threads/{threadId}/stackFrames
-------------------------------------------------------------------*/

/*
*
* @success 200 Returns list of stack frames for thread
* @error 404 if VM is not running or thread is not found or cannot be suspended
* @format application/json or text/html depending on Accept header
*
*/
def onList() {
	def appId = request.params.applicationsId[]
    def threadId = request.params.threadsId[]
    threadId = new Integer(threadId)
	
	def framesList
	if (0 == threadId) {
        // PHP
        //  get PHP debugger
        // TODO should there be error handling for PHP like there is for Java/Groovy?
        def phpThreadService = Framework.createServiceContext().getService('phpthreads')
        if (phpThreadService) {
            // TODO
            def thread = phpThreadService.getThread(appId, threadId)
            framesList = thread.getStackFrames()
        }
	} else {
	    // Java
	    def debugger = DebugUtils.getDebugger()
	    if(null==debugger) {
	        logger.FINE {DebugUtils.DEBUG_NULL_ERR}
	        request.status = 404
	        return
	    }
	    def debugVM = debugger.getVM()
	    if (null == debugVM) {
	        logger.FINE {DebugUtils.VM_NULL_ERR}
	        request.status = 404
	        return
	    }
	    def thread = DebugUtils.getThreadReference()
	    if (null == thread) {
	        logger.FINE {DebugUtils.THREAD_NULL_ERR}
	        request.status = 404
	        return
	    }
	    framesList = DebugUtils.getFrames(thread)
	}
    
    returnJsonObject(framesList)
}
    
def returnJsonObject(object) {
	logger.FINE {"Returning JSON object"}
	request.json.output = object
    request.view = "JSON"
    render();
    request.status = 200
}

/*
*
* @success 200 Returns stack frame info
* @error 404 if VM is not running or thread is not found or cannot be suspended
* @format application/json or text/html depending on Accept header
*
*/
def onRetrieve() {
    
    def appId = request.params.applicationsId[]
    def threadId = request.params.threadsId[]
    threadId = new Integer(threadId)
    def frameIndex = request.params.stackFramesId[]
    frameIndex = new Integer(frameIndex)
    
    def frame
    if (threadId == 0) {
        // PHP
        //  get PHP debugger
        // TODO should there be error handling for PHP like there is for Java/Groovy?
        def phpThreadService = Framework.createServiceContext().getService('phpthreads')
        if (phpThreadService) {
            // TODO
            def thread = phpThreadService.getThread(appId, threadId)
            def frames = thread.getStackFrames()
            frame = frames[frameIndex]
        }
    } else {
        // Java
        def debugger = DebugUtils.getDebugger()
        if(null==debugger) {
            logger.FINE {DebugUtils.DEBUG_NULL_ERR}
            request.status = 404
            return
        }
        def debugVM = debugger.getVM()
        if (null == debugVM) {
            logger.FINE {DebugUtils.VM_NULL_ERR}        
            request.status = 404
            return
        }
            
        def thread = DebugUtils.getThreadReference()
        if (null == thread) {
            logger.FINE {DebugUtils.THREAD_NULL_ERR}
            request.status = 404
            return
        }
    
        
        if (null == frameIndex) {
            logger.FINE {DebugUtils.FRAME_NULL_ERR}
            request.status = 404
            return
        }
       
        frame = DebugUtils.getFrameRepresentation(thread, frameIndex)
    }

    returnJsonObject(frame)
}

