/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 *  WebSphere sMash
 *
 * (C) Copyright IBM Corp. 2007  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */

/*
 * This code is invoked by processes.groovy during process
 * creation and deletion.
 */

import zero.ide.services.AppUtils 
import zero.ide.services.BreakpointManager 
import zero.ide.services.Debugger
import zero.ide.services.DebugUtils
import zero.management.appconfig.Application

def onPreStart() {
	def debug = zget("/event/debug")
	// check if the process is being debugged
	if(!debug) {
		return
	}	
	String appId = zget("/event/appId")
	List<String> args = zget("/event/args")
	
	def debugAddress = findUnusedAddress()
    startDebug(debugAddress, appId)
    args << getMaxRequestsArg(appId)
    args << "-jvmarg=\"-agentlib:jdwp=transport=dt_socket,address="+debugAddress[0]+":"+debugAddress[1]+",timeout=5000\""
}

def startDebug(def address, def appId) { 
	def vmm = org.eclipse.jdi.Bootstrap.virtualMachineManager()
	def debugger = zget(Debugger.getDebuggerGcUri(appId))
	if(debugger!=null) {
		debugger.stop()
	}
	List lcs = vmm.listeningConnectors()
	for(com.sun.jdi.connect.ListeningConnector lc: lcs) {
		if(lc.transport().name().equals("dt_socket")) {
			logger.FINE {DebugUtils.CONN_FOUND}
			def args = lc.defaultArguments()
			com.sun.jdi.connect.Connector.Argument portArg = args.get("port")
			portArg.setValue(address[1])
			args.put("port",portArg)
			logger.FINE {"Starting debug listener on port " + address[1]}
			address = lc.startListening(args)
			debugger = new Debugger(lc, args, appId)
			zput(Debugger.getDebuggerGcUri(appId), debugger)
			new Thread(debugger).start()
			sleep(1000)
			return
		}		
	}
}

def onPreStop() {
	String appId = zget("/event/appId")
	def debugger = zget(Debugger.getDebuggerGcUri(appId))
	if(debugger!=null&&debugger.getVM()!=null) {
		BreakpointManager.setSkipBreakpoints(appId, true)
		def allThreads = debugger.getVM().allThreads() 
		allThreads.each { thread->
			if(thread.isSuspended()) {
				DebugUtils.resume(debugger, thread)		
			}
		}		
	}	
}

def onPostStop() {
	String appId = zget("/event/appId")
	def debugger = zget(Debugger.getDebuggerGcUri(appId))
	if(debugger!=null) {
		debugger.stop()
		zdelete(Debugger.getDebuggerGcUri(appId))
	}	
}

/**
 * Return a [ipAddress (String), port (integer)] array for an 
 * open address / port on this system
 */
def findUnusedAddress() {
    def socket = new java.net.ServerSocket()
    def socketAddress = new InetSocketAddress(0)
    socket.bind(socketAddress)
    socketAddress = socket.getLocalSocketAddress()
    def hostAddress = socketAddress.getAddress().getHostAddress()
    if (hostAddress.equals("0.0.0.0")) hostAddress = "127.0.0.1"
    def result = [hostAddress, socketAddress.getPort()]    
    socket.close()
    // Return a static address until we can pass JVM args to 'zero start'
    // see http://www.projectzero.org/bugzilla/show_bug.cgi?id=5714
    //return ["127.0.0.1", "8099"]
    return result
}



def getMaxRequestsArg(appId) {	
	def app = Application.getApplication(AppUtils.getApphome(appId))
	def mode = app.zgetConfigValue("/config/runtime/mode")
	def maxReqArg = "-G=\"/config/zso/recycle/"
	if(mode!=null) {
		maxReqArg += mode + "/"
	} 
	return maxReqArg + "maxRequests=-1\"" 
}
