/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 *  WebSphere sMash
 *
 * (C) Copyright IBM Corp. 2007,2008  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */
dojo.provide("zero.ide.widget.debug.DebugView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Menu");
dojo.require("dijit.Tree");
dojo.require("dijit.layout.SplitContainer");
dojo.require("dijit.layout.StackContainer");
dojo.require("dijit.layout.TabContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.Button");
dojo.require("zero.ide.widget.editor.CodeViewer");
dojo.require("zero.ide.data.LazyLoadReadStore");
dojo.require("zero.ide.widget.debug.DebugGutterMenu");

dojo.declare("zero.ide.widget.debug.BreakpointsTreeModel", dijit.tree.ForestStoreModel, {
    mayHaveChildren: function(/*dojo.data.Item*/ item) {
        return false;
    }
});

dojo.declare("zero.ide.widget.debug.VariablesTreeModel", dijit.tree.ForestStoreModel, {
    mayHaveChildren: function(/*dojo.data.Item*/ item) {
        return item.isObject;
    }
});

dojo.declare("zero.ide.widget.debug.ThreadsTreeModel", dijit.tree.ForestStoreModel, {
    mayHaveChildren: function(/*dojo.data.Item*/ item) {
        if(item.root==true) {
            return true;
        }
        if(item.isStackFrame == true) {
           return false;
        }
        if(item.url.indexOf("stackFrames") == -1) {
            if(item.isSuspended) {
                return true;
            }
        }
        return false;
    }
});

dojo.declare(
    "zero.ide.widget.debug.DebugView",
    [dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("zero","ide/widget/debug/templates/DebugView.html"),
		widgetsInTemplate: true,
		
		appId: null,
        breakpoints: [],    
		fileExplorerWidget: null,
		fileExplorerStore: null,
		fileExplorerModel: null,
		variablesWidget: null,	
		variablesStore: null,
		variablesModel: null,
		threadsWidget: null,	
		threadsStore: null,
		threadsModel: null,
		threadId: null,	
		stackFrame: null,
		lastMatchedSourcePath: null,
		codeViewerGutter: null,
		codeViewerWidget: null,
		_selectedFileItem: null,
		gutterMenu: null,
		lastAction: null,
		timer: null,
        debugExts: [".groovy", ".java", ".php"],
        Etag: null,
        lastUpdateRequest: 0,
        POLL_INTERVAL: 5000,
        BP_VALIDATION_FREQ: 5,
        numUpdates: 0,
        executableLines: [],
		
		handleStackInfo: function(topStackFrame, args) { 
    	    if(topStackFrame==null) {
    	    	this.updateThreadNotSuspended();
    	    	return;
    	    }
		    this.stackFrame = topStackFrame;
		    var normalizedPath = this.stackFrame.sourcePath.replace("\\", "/");
            if(this.codeViewerWidget!=null && this.codeViewerWidget.filePath.indexOf(normalizedPath)!=-1) {
              // file is already being viewed so just set the cursor
              this.codeViewerWidget.setDebugCursor(this.stackFrame.lineNumber);
              this.codeViewerWidget.onGotoLine(this.stackFrame.lineNumber);             
            } else {            
    		    // search for files that match path
    		    dojo.xhrGet( {
    	        	url: "resources/applications/"+this.appId+"/files?metadata&recurse&flat&nodirs&sort&hide&filter="+topStackFrame.sourcePath,
    	        	handleAs: "json",
    	        	load: dojo.hitch(this, this.handleMatchedFilesInfo),	        	
    	        	error: dojo.hitch(this, function(type, error) {
    	        	   if(type.status==404) { 
    	        	      // then no matches
    	        	   } else {
    	        	      alert("Error in handleStackInfo()"); 
    	        	   }        	   
    	        	})        	
    			});	
            }   			   		    
		},
		
		displaySourceNotFound: function(path) {
			if (this.codeViewerWidget == null) {
				if(zlog.isDebug())console.debug("DebugView: initialize code viewer");
				this.codeViewerWidget = new zero.ide.widget.editor.CodeViewer(
					{
						filePath: path,
						//title: path  // dont show filepath as a tooltip/title
					},
					this.debugView$CodeViewer
				);
				this.codeViewerWidget.startup();
				dojo.connect(this.debugView$CodeViewerPane,"resize",this.codeViewerWidget,"refresh");
			}
	        this.codeViewerWidget.clearDebugCursor();
            this.codeViewerWidget.clearBreakpoints();
            this.codeViewerWidget.setFilePath("");
            this.codeViewerWidget.setData("Cannot find source file for " + path + ".\nClick 'Resume' or 'Step Out' to return to a line that has locatable source code.\nRefer to the documentation for more information on how to change the step filters for the application builder so that non-locatable source can be skipped.");			            			        
		},	
		
		handleMatchedFilesInfo: function(matches) {
		    if(matches.length <= 0) {
		        // cannot find file
		    	this.displaySourceNotFound(this.stackFrame.sourcePath);
		        return;
		    }
		    var prefix = "resources/applications/";
		    var index = matches[0].path.indexOf(prefix) + prefix.length;
		    prefix = "files/";
		    index = matches[0].path.indexOf(prefix,index) + prefix.length;
		    var title = matches[0].path.substring(index);
		    this.lastMatchedSourcePath = matches[0].path;
		       		    
		    if(this.codeViewerWidget==null || this.codeViewerWidget.filePath!=matches[0].path) {
		    	this.loadFileInCodeViewer(matches[0].path,title);
		    } else {
		    	this.codeViewerWidget.onGotoLine(this.stackFrame.lineNumber);
		    }	        
		    this.setBreakpointsInSourceView();
		    this.codeViewerWidget.setDebugCursor(this.stackFrame.lineNumber);
		},
		
		updateExecutableLines: function() {
			var srcPath = this.getRelativeSourcePath(this.codeViewerWidget.filePath);
			if(this.isFileDebuggable(srcPath)) {
				dojo.xhrGet( {
		        	url: "resources/applications/"+this.appId+"/sourceDetails/"+srcPath+"/executables",
		        	handleAs: "json",
		        	preventCache: true,
		        	load: dojo.hitch(this, function(data, error) {
		        	   this.executables = data;        	   
		        	}),
		        	error: dojo.hitch(this, function(response, ioArgs) {
						var manager = dijit.byId('app-mgr-ui-id');					
						var errMsg = eval('(' + ioArgs.xhr.responseText + ')').errorMessage;
						var formattedMessage = '';
						//determine is its an array object before formatting message
						if ( errMsg.constructor == Array){
							for(var i = 0; i < errMsg.length; i++){
								formattedMessage += errMsg[i]+ "<br />";
							}
						}else{
							formattedMessage = (errMsg.replace(/\r\n/g, "<br />")).replace(/\n/g, "<br />");
						}
						manager._statusDialog.showDialog('Compile Error', '', formattedMessage );     	   
		        	})        	
				});
			} else {
				this.executables = [];
			}						
		},
		
		canSetBreakpointOnLine: function(line) {
			// until there is a way to check for executable lines in java, 
			// always return true for java source and php
			if((this.codeViewerWidget.filePath.indexOf('.java') > 0) ||
			   (this.codeViewerWidget.filePath.indexOf('.php') > 0)) {
				return true;
			}
			return this.isExecutable(line);
		},
		
		isExecutable: function(line) {
			if(this.executables == null) {
				return false;
			}
			for(var i = 0; i < this.executables.length; i++) {
    			if(this.executables[i]==line) {
    				return true;
    			}
    		}
			return false;
		},
		
		fetchSourceInfo: function() {
		    if(this.threadId==null) {
		       // not suspended
		    	this.updateThreadNotSuspended();
		       return;
		    }
		    dojo.xhrGet( {
	        	url: "resources/applications/"+this.appId+"/threads/"+this.threadId+"/stackFrames/0",
	        	handleAs: "json",
	        	preventCache: true,
	        	load: dojo.hitch(this, this.handleStackInfo),
	        	error: dojo.hitch(this, function(type, error) {
	        	   if(type.status==404) { 
	        	      // then the VM is probably not running
	        	      this.updateThreadNotSuspended();
	        	   } else {
	        	      // alert("Server error in fetchSourceInfo()"); 
	        	   }        	   
	        	})        	
			});
		},
		
        /* If not threads are suspended then indicate it in the variables view.
           Else if the thread that was already being viewed is suspended then show its state in the views.
           Else show the first suspended thread in the views.
        */
		updateThreads: function(threads) {
		   this.displayThreads(threads);
           var threadIndex = -1;
		   for(var i = 0; i < threads.length; i++) {
	   	        if(threads[i].isSuspended == true) {
                    if(threadIndex==-1) {
                        threadIndex = i;
                    } else if(threads[i].threadId==this.threadId) {
                        threadIndex = i;
                    }                   
	   	        }
	   	   }
		   if(threadIndex == -1) {
		       // there is no suspended thread, so lets disable step/resume buttons
		       this.updateThreadNotSuspended();
		   } else {
               this.updateSuspendedThread(threads[threadIndex]);
		   }		   
		},
		
		updateThreadNotSuspended: function() {
		    // there is no suspended thread, so lets disable step/resume buttons
           this.stackFrame = null;
		   this.resetVariablesView();
	       this.printNotSuspendedMessage();
	       this._enableButtons(false);
	       if(this.codeViewerWidget != null ) {	           
	           this.codeViewerWidget.clearDebugCursor();
	       }	       
		},
		
		displayThreads: function(threads) {		    
		   var url = "resources/applications/"+this.appId+"/threads";
		   this.resetThreadsView();
		   this.threadsStore = new zero.ide.data.LazyLoadReadStore(
				{
					url: url, 
					identifierAttr: "url", 
					labelAttr: "label"	
				}, 
				true /*preventCache*/
			);
		   this.threadsModel = new zero.ide.widget.debug.ThreadsTreeModel (
		      {
		      	"id":"threadsTreeModelId",
				"jsId":"jsThreadsTreeModelId",
				"store": this.threadsStore
		      } 
		   );		    	
		    var div = document.createElement("div");
			this.threadsContainer.domNode.appendChild(div);
			this.threadsWidget = new dijit.Tree(
			 {
			    "appId": this.appId,
			    "labelAttr": "label",
	            "showRoot": false,
	            "isExpanded": false,
	            "model": this.threadsModel			 },
			 div 
			);
			dojo.connect(this.threadsWidget,"focusNode",this,"threadTreeSelected");
		},
		
		threadTreeSelected: function(node) {
		    this.threadId = node.item.threadId;
            // if we have a suspended thread or a stack frame
		    if(node.item.isSuspended == true || node.item.lineNumber) {
		        this.loadVariablesView();
		        this.fetchSourceInfo();
                this._enableButtons(true);
		    } else {
		        this.updateThreadNotSuspended();
                this._enableButtons(false);
		    }
		},
		
		updateSuspendedThread: function(thread) {
            this.threadId = thread.threadId; 
            if(thread.isSuspended == false ) {
               this.updateThreadNotSuspended();
		       return;
            }	
		   // there is a suspended thread, so lets update the source view and enable the step/resume buttons 
		   this.fetchSourceInfo();
		   this._enableButtons(true);
		   this.loadVariablesView();		      	   
		},
		
		printNotSuspendedMessage: function() {
		    this.variablesContainer.domNode.innerHTML = "<p>The selected thread is not suspended.  Select a suspended thread to view its variables.</p>";
		},
		
		loadVariablesView: function() {
		   var url = "resources/applications/" + this.appId + "/threads/" + this.threadId + "/stackFrames/0/localVariables";
		   this.resetVariablesView(); 
		   this.variablesStore = new zero.ide.data.LazyLoadReadStore(
				{
					url: url, 
					identifierAttr: "url", 
					labelAttr: "summary"	
				}, 
				true /*preventCache*/
			);
		   this.variablesModel = new zero.ide.widget.debug.VariablesTreeModel (
		      {
		      	"id":"varTreeModelId",
				"jsId":"jsVarTreeModelId",
				"store": this.variablesStore
		      } 
		   );		    	
		    var div = document.createElement("div");
			this.variablesContainer.domNode.appendChild(div);
			this.variablesWidget = new dijit.Tree(
			 {
			    "appId": this.appId,
			    "labelAttr": "summary",
	            "showRoot": false,
	            "isExpanded": false,
	            "model": this.variablesModel
			 },
			 div 
			);
		},
        
		resetVariablesView: function() {
		   while(this.variablesContainer.domNode.childNodes[0]) {
        	   this.variablesContainer.domNode.removeChild(this.variablesContainer.domNode.childNodes[0]);
	       }
		   if (this.variablesWidget != null) {		        
				this.variablesWidget.destroyRecursive();
				this.variablesModel.destroy();				
				this.variablesWidget = null;
		   }
		},
        
        resetBreakpointsView: function() {
            this.breakpoints = [];
        },
		
		resetThreadsView: function() {
    	   while(this.threadsContainer.domNode.childNodes[0]) {
        	   this.threadsContainer.domNode.removeChild(this.threadsContainer.domNode.childNodes[0]);
           }
     
		   if (this.threadsWidget != null) {
                this.threadsWidget.destroyRecursive();
                this.threadsModel.destroy();				
                this.threadsWidget = null;
		   }
		},
	
		updateDebugState: function(forceRefresh) {		   	
		   this.lastUpdateRequest = (new Date()).getTime();
		   var bindArgs = {
			    url: "resources/applications/"+this.appId+"/threads",
	        	handleAs: "text",
	        	preventCache: true,
	        	load: dojo.hitch(this, function(data, args) {
	        		this._setEtag(args);
	        		if (args.xhr.status != 304) { //check if modified
	        		    this.updateThreads( eval('('+data+')') );
	        		}
	        	}),
	        	error: dojo.hitch(this, function(type, error) {
	        	   if(type.status==404) {
	        	      // then the VM is probably not running
	        	      this.updateThreadNotSuspended();
                      this.resetThreadsView();
	        	   }         	   
	        	})
		   }
		   if(!forceRefresh && this.Etag!=null) {
			   bindArgs.headers = {"If-None-Match": "\""+this.Etag+"\""};
		   }
		   dojo.xhrGet(bindArgs);
		}, 			
		
		_setEtag: function(args) {
			var joinheaders = args.xhr.getAllResponseHeaders();
			var headers = joinheaders.split("\n");
			for (var i=0; i < headers.length; i++) {
				var etagpos = headers[i].indexOf("Etag:");
				if (etagpos < 0) {
					etagpos = headers[i].indexOf("ETag:");
				}
				if (etagpos == 0) {
					this.Etag = headers[i].substr(headers[i].indexOf("\"")+1,headers[i].lastIndexOf("\"")-headers[i].indexOf("\"")-1);	
				}
			}
		},
			
		getApplicationId: function () {
			return this.appId;
		},			
		
		startup: function(){
			this.inherited("startup",arguments);			
		},
        
		opening: function(appId) {
            dojo.connect(dijit.byId('app-mgr-ui-id')._controller,"setStatusRunning",this,"onAppStatusChanged");
			if (appId == null) {
				alert("ERROR: Cannot open debug view without an application id.");
				return;
			}			

			if (appId != this.appId) {
				this.appId = appId;
				this.clearViewer();
			} else if(this.codeViewerWidget!=null) {
				// search on file name and check for a match 				
   	        	//  commented out url statement...won't work because the path will have /vFiles/ or /files/, therefore, search on fileName 
				//url: "resources/applications/"+this.appId+"/files?metadata&recurse&flat&nodirs&sort&hide&filter="+this.codeViewerWidget.filePath,
    		    dojo.xhrGet( {
    	        	url: "resources/applications/"+this.appId+"/files?metadata&recurse&flat&nodirs&sort&hide&filter="+
					       this._selectedFileItem._fullName.substring(this._selectedFileItem._fullName.lastIndexOf('/')+1),
    	        	handleAs: "json",
    	        	load: dojo.hitch(this, function(matches, args) {
						if ( this._containsValue(matches,this._selectedFileItem._fullName)){
							this.codeViewerWidget.onOpen();
    	    		    }else{
    	    		        // cannot find file
    	        			// clear the code viewer since the file no longer exists
    	    		    	this.clearViewer();    	    		        
    	    		    }
    	        	}),	        	
    	        	error: dojo.hitch(this, function(type, error) {})        	
    			});
			}
			
			this.checkAppStatus();

			// For safety purposes, always rebuild the tree.  We don't know how the
			// content may have been changed by another part of the App Builder
			this._rebuildTree(this.checkbox.checked);
			
			// This is necessary for the SplitContainer to show.  Seems like a dojo bug.
			this.resize();
            
            this.refreshAll();
            
            this.pollForUpdate()
		},
		
		checkAppStatus: function() {
			dojo.xhrGet( {
	        	url: "resources/applications/"+this.appId,
	        	handleAs: "json",
	        	sync: false,
	        	load: dojo.hitch(this, function(status, args) {
	        		if (status.isRunnable && status.isRunning && !status.isDebugging) {
	        			//alert("The application status is invalid.  This is probably because the application was started outside AppBuilder.  Before you can debug the application, you will need to restart it.");
						var manager = dijit.byId('app-mgr-ui-id');					
						var newLine = "<br>"; // newline
						var formattedMessage = "This is probably because the application was started outside of the App Builder Debug tab."+newLine+"Before you can debug the application, you will need to stop it and restart it from the Debug tab."
						manager._statusDialog.showDialog('Debug not available', 'The application is not running in debug mode.', formattedMessage );     	   

	        		}					
	        	}),	        	
	        	error: dojo.hitch(this, function(type, error) {})        	
			});			
		},
		
		isDebugTabSelected: function() {
			return dijit.byId("mainTabContainer").selectedChildWidget.id == "debugView";
		},
		
		pollForUpdate: function() {
			setTimeout(dojo.hitch(this,this.pollForUpdate), this.POLL_INTERVAL);
			// if we have not requested an update recently enough, then do it now if debug tab is selected
			if(this.isDebugTabSelected() && (new Date()).getTime() - this.lastUpdateRequest > this.POLL_INTERVAL) {
				this.updateDebugState(false);
				if((this.numUpdates%this.BREAKPOINT_VALIDATION_FREQ)==0) {
					this.validateBreakpoints();
				}
			}
			this.numUpdates++;
		},
		
		validateBreakpoints: function() {
			var deletedFiles = [];
			var validatedFiles = [];
			var bpsToRemove = [];
			for(var i = 0; i < this.breakpoints.length; i++) {
				for(var j = 0; j < deletedFiles.length; j++) {
					if(deletedFiles[i]==this.breakpoints[i].sourcePath) {
						bpsToRemove << i;
					}
				}
				var validated = false;
				for(var j = 0; j < validatedFiles.length; j++) {
					if(validatedFiles[i]==this.breakpoints[i].sourcePath) {
						validated = true;
					}
				}
				if(!validated) {
					var paramsObj = {  
						url: "resources/applications/"+this.appId+"/files/"+this.breakpoints[i].sourcePath+"?metadata&recurse&flat&nodirs&sort&hide",
			        	sync: false,   
				    	load: dojo.hitch(this, function(matches, args) {
				    		validatedFiles << this.breakpoints[i].sourcePath; 
			        	}),	
				    	error: dojo.hitch(this, function(type, error) {        	   
		        	        deletedFiles <<  this.breakpoints[i].sourcePath;
		        	        bpsToRemove << i;
			        	}),	
		            	contentType: "application/json",
						mimetype: "application/json",
		            	handleAs: 'json'		
					}		
					dojo.xhrGet(paramsObj);
				}
			}			
			for(var i = 0; i < bpsToRemove.length; i++) {
				this.removeBreakpoint(this.breakpoints[bpsToRemove[i]].sourcePath, this.breakpoints[bpsToRemove[i]].line);
			}
		},
        
        onAppStatusChanged: function(bRunning, bRunnable) {
			if(this.isDebugTabSelected()) {
				// user clicked stop or start and application has changed state
	            if(bRunning) {
	                if(this.threadsWidget==null) {
	                    this.updateDebugState(true);
	                }                
	            } else {
	                this.resetVariablesView();
	                this.resetThreadsView();
	            }
			}            
        },
        
		attachGutterMenu: function() {
		    if(this.gutterMenu==null) {
                this.codeViewerWidget.lineNode.setAttribute("id","debugView$CodeViewerGutter");                
	            this.gutterMenu = new zero.ide.widget.debug.DebugGutterMenu({targetNodeIds:["debugView$CodeViewerGutter"], id:"debugView$CodeViewerGutterMenu", debugView: this});
	            this.gutterMenu.addSetCommand(new dijit.MenuItem({label:"Set breakpoint", onClick: dojo.hitch(this,this.onSetBreakpoint)}));
	            this.gutterMenu.addClearCommand(new dijit.MenuItem({label:"Clear breakpoint", onClick: dojo.hitch(this,this.onClearBreakpoint)}));
	            this.gutterMenu.addEnableCommand(new dijit.MenuItem({label:"Enable breakpoint", onClick: dojo.hitch(this,this.onEnableBreakpoint)}));
	            this.gutterMenu.addDisableCommand(new dijit.MenuItem({label:"Disable breakpoint", onClick: dojo.hitch(this,this.onDisableBreakpoint)}));
	            this.gutterMenu.startup();
	         }		    
		},		
		
		onDblClickGutter: function(evt) {
		    var codeViewerPath = this.getRelativeSourcePath(this.codeViewerWidget.filePath);
			var line = this.codeViewerWidget.getSourceLine(evt.clientY);
			if(this.isPhpScript(codeViewerPath) || this.isExecutable(line)) {
	            if(this.isBreakpointSet(codeViewerPath, line)) {
	            	this.clearBreakpoint(codeViewerPath, line);
	            } else {
	            	this.setBreakpoint(codeViewerPath, line);
	            }
			}			
		},
        
        detachGutterMenu: function() {
			if(this.gutterMenu!=null) {
				this.gutterMenu.destroyRecursive();
	            this.gutterMenu = null;
			}
        },
        
        isBreakpointSet: function(srcPath, line) {
            for(var i = 0; i < this.breakpoints.length; i++) {
                if(this.breakpoints[i].sourcePath==srcPath && this.breakpoints[i].line==line) {
                    return true;
                }
            }
            return false;
        },
        
        onSetBreakpoint: function(evt) {
            var srcPath = this.getRelativeSourcePath(this.codeViewerWidget.filePath);
        	this.setBreakpoint(srcPath, this.gutterMenu.line);
        },
				
		setBreakpoint: function(srcPath, line) {            
            if(this.isBreakpointSet(srcPath, line)) {
                return;
            }
		    this.codeViewerWidget.addBreakpoint(line, true);
            this.addBreakpoint(srcPath, line, true);
		   	var breakpoint = {
				sourcePath: srcPath,
				line: line,
                enabled: true
    	    };      	   
    	   
    	    var paramsObj = {  
	        	url: "resources/applications/"+this.appId+"/breakpoints",
	        	sync: false,   
		    	load: dojo.hitch(this, function(data, args) { 
	        	}),	
		    	error: dojo.hitch(this, function(type, error) {        	   
        	      alert("Server error in adding breakpoint"); 
	        	}),	
            	contentType: "application/json",
				mimetype: "application/json",
				postData: dojo.toJson(breakpoint),
            	handleAs: 'text'		
			}		
			dojo.rawXhrPost(paramsObj);		   
		},
        
        addBreakpoint: function(sourcePath, line, enabled) {
            var breakpointsTable = dojo.byId("BreakpointsTable");
            var newBreakpoints = [];
            var inserted = false;
            for(var i = 0; i < this.breakpoints.length + 1; i++) {
                if(i==this.breakpoints.length || sourcePath < this.breakpoints[i].sourcePath || (sourcePath == this.breakpoints[i].sourcePath && line < this.breakpoints[i].line)) {
                    var row = breakpointsTable.insertRow(i);
                    var cell = row.insertCell(row.cells.length);
                    var deleteDiv = document.createElement('div');
                    cell.appendChild(deleteDiv);
                    var deleteBtn = new dijit.form.Button(
                    	{
                    		id: 'breakpointDelete' + ':' + sourcePath + ':' + line,
                    		showLabel: false,
                    		label: 'Delete breakpoint in ' + sourcePath + ', line ' + line,
                    		iconClass: 'dijitRemoveBreakpointButtonIcon',
                    		baseClass: 'breakpointDeleteButton'
                    	},
                    	deleteDiv
                    );
                    dojo.connect(deleteBtn, 'onClick', this, 'onDeleteBreakpointFromMgr');
                    
                    cell = row.insertCell(row.cells.length);
                    var boxDiv = document.createElement('div');
                    cell.appendChild(boxDiv);
                    var id = sourcePath + ':' + line;
                    var bpBox = new dijit.form.CheckBox(
                     {                
                        checked: enabled,
                        id: id
                     },
                     boxDiv 
                    );
                    dojo.connect(bpBox, 'onClick', this, 'onBreakpointCheckChanged');
                    cell = row.insertCell(row.cells.length);
                    var text = document.createTextNode(sourcePath+' [line: '+line+']');
                    cell.appendChild(text);
                    
                    var bp = { sourcePath: sourcePath, line: line, enabled: enabled, box: bpBox, deleteBtn: deleteBtn };
                    newBreakpoints.push(bp);
                    break;
                } else {
                    newBreakpoints.push(this.breakpoints[i]);
                }                
            }             
            for(var j = i; j < this.breakpoints.length; j++) {
                newBreakpoints.push(this.breakpoints[j]);
            }  
            this.breakpoints = newBreakpoints;                        
        },
        
        onDeleteBreakpointFromMgr: function(evt) {
        	var args = evt.target.id.split(':');
        	this.clearBreakpoint(args[1], args[2]);
        },
        
        clearBreakpoints: function() {
            var breakpointsTable = dojo.byId("BreakpointsTable");
            for(var i = 0; i < this.breakpoints.length; i++) {
                this.breakpoints[i].box.destroyRecursive();
                this.breakpoints[i].deleteBtn.destroyRecursive();
                breakpointsTable.deleteRow(0);                
            }
            this.breakpoints = [];
        },
        
        removeBreakpoint: function(sourcePath, line) {
            var breakpointsTable = dojo.byId("BreakpointsTable");
            var newbreakpoints = [];
            for(var i = 0; i < this.breakpoints.length; i++) {
                if(this.breakpoints[i].sourcePath==sourcePath && this.breakpoints[i].line==line) {
                    this.breakpoints[i].box.destroyRecursive();
                    this.breakpoints[i].deleteBtn.destroyRecursive();
                    breakpointsTable.deleteRow(i);
                } else {
                    newbreakpoints.push(this.breakpoints[i]);
                }
            }            
            this.breakpoints = newbreakpoints;
        },
        
        isBreakpointEnabled: function(file, line) {
        	for(var i = 0; i < this.breakpoints.length; i++) {
        		if(this.breakpoints[i].sourcePath==file && this.breakpoints[i].line==line) {
        			return this.breakpoints[i].enabled;
        		}
        	}
        	return false;
        },	
        
        onBreakpointCheckChanged: function(evt) {
            var args = evt.target.id.split(':');
            this.updateBreakpoint(args[0],args[1],evt.target.checked);
        },
        
        updateBreakpoint: function(srcPath,line,enabled) {
        	this.enableBreakpoint(srcPath,line,enabled);
        	for(var i = 0; i < this.breakpoints.length; i++) {
        		if(this.breakpoints[i].sourcePath==srcPath && this.breakpoints[i].line==line) {
        			this.breakpoints[i].box.setValue(enabled);
        			this.breakpoints[i].enabled = enabled;
        			break;
        		}
        	}        	
        },
        
        onEnableBreakpoint: function(evt) {
        	this.updateBreakpoint(this.getRelativeSourcePath(this.codeViewerWidget.filePath),this.gutterMenu.line,true);        	
        },
        
        onDisableBreakpoint: function(evt) {
        	this.updateBreakpoint(this.getRelativeSourcePath(this.codeViewerWidget.filePath),this.gutterMenu.line,false);
        },
        
        enableBreakpoint: function(sourcePath,line,enabled) {        	
            if(this.getRelativeSourcePath(this.codeViewerWidget.filePath)==sourcePath) {
                this.codeViewerWidget.enableBreakpoint(line, enabled);
            }            

            var putD = {
                enabled: enabled 
            }; 
            
            var paramsObj = {  
                url: "resources/applications/"+this.appId+"/breakpoints/"+sourcePath+"/"+line,
                sync: false,   
                load: dojo.hitch(this, function(data, args) { 
                }), 
                error: dojo.hitch(this, function(type, error) {            
                  alert("Error enabling/disabling breakpoint"); 
                }), 
                contentType: "application/json",
                mimetype: "application/json",
                putData: dojo.toJson(putD),
                handleAs: 'text'        
            };       
            dojo.rawXhrPut(paramsObj);
        },
        
        onSkipAllBreakpointsChanged: function(evt) {
            var putD = {
                skipAll: evt.target.checked 
            }; 
            
            var paramsObj = {  
                url: "resources/applications/"+this.appId+"/breakpoints",
                sync: false,   
                load: dojo.hitch(this, function(data, args) { 
                }), 
                error: dojo.hitch(this, function(type, error) {            
                  alert("Error setting breakpoints to 'skip all'"); 
                }), 
                contentType: "application/json",
                mimetype: "application/json",
                putData: dojo.toJson(putD),
                handleAs: 'text'        
            };       
            dojo.rawXhrPut(paramsObj);
        },
        
        onClearBreakpoint: function(evt) {
        	this.clearBreakpoint(this.getRelativeSourcePath(this.codeViewerWidget.filePath), this.gutterMenu.line);        	
        },
               
        clearBreakpoint: function(sourcePath, line) {
           if(this.codeViewerWidget!=null && this.getRelativeSourcePath(this.codeViewerWidget.filePath)==sourcePath) {
        	   this.codeViewerWidget.clearBreakpoint(line);
           }   		   
           this.removeBreakpoint(sourcePath, line);

           var paramsObj = {  
	        	url: "resources/applications/"+this.appId+"/breakpoints/"+sourcePath+"/"+line,
		    	load: dojo.hitch(this, function(data, args) {        	   
	        	}),	
		    	error: dojo.hitch(this, function(type, error) {        	   
        	      alert("Server error in removing breakpoint"); 
	        	}),	
            	handleAs: 'text'		
			}		
			dojo.xhrDelete(paramsObj);
		},		
		
		fetchBreakpoints: function() {
		    dojo.xhrGet( {
	        	url: "resources/applications/"+this.appId+"/breakpoints",
	        	handleAs: "json",
	        	headers: {"Accept": "application/json"},
	        	load: dojo.hitch(this, this.loadBreakpoints),
	        	error: dojo.hitch(this, function(type, error) {
	        	   if(type.status==404) { 
	        	      // no breakpoints for this source file
	        	   } else {
	        	      alert("Server error in fetchBreakpoints()");
	        	   }        	               	   
	        	})        	
			});	
		},
		
		loadBreakpoints: function(breakpoints, args) {
            this.clearBreakpoints();
            var codeViewerPath = null;
            if(this.codeViewerWidget!=null) {
            	codeViewerPath = this.getRelativeSourcePath(this.codeViewerWidget.filePath);
            }
		    for(var i = 0; i < breakpoints.length; i++) {
		    	if(codeViewerPath!=null && codeViewerPath==breakpoints[i].sourcePath) {
                    this.codeViewerWidget.addBreakpoint(breakpoints[i].line,breakpoints[i].enabled);
                }		        
                this.addBreakpoint(breakpoints[i].sourcePath,breakpoints[i].line,breakpoints[i].enabled);
		    }
		},		
        
        setBreakpointsInSourceView: function() {
			var codeViewerPath = null;
			if(this.codeViewerWidget!=null) {
				codeViewerPath = this.getRelativeSourcePath(this.codeViewerWidget.filePath);
			}
            for(var i = 0; i < this.breakpoints.length; i++) {
               if(codeViewerPath!=null && this.breakpoints[i].sourcePath==codeViewerPath) {
                    this.codeViewerWidget.addBreakpoint(this.breakpoints[i].line,this.breakpoints[i].enabled);
                }
            }
        },
		
		resize: function(contentbox) {
			var args = null;
			try {
				if (!contentbox || !contentbox.h) {
					contentbox = dojo.contentBox(this.domNode);
				}
				//var upperbox = dojo.marginBox(this.toolbarNode);
				//args = { h: contentbox.h - upperbox.h };
				args = { h: contentbox.h - 6};  // avoid double scrollbars
			} catch(e) {
				if(zlog.isDebug())console.debug("DebugView: resize, Error finding size of lotus elements. "+e);
			}
			this.split.resize(args);
		},
				
		_rebuildTree: function(virtual) {						
			if (this.fileExplorerWidget != null) {
				//this.fileExplorerWidget.dndController.destroy(); later will use DnD feature								
				this.fileExplorerWidget.destroyRecursive();
				this.fileExplorerModel.destroy();				
			}
			
			this.checkbox.attr('checked',virtual);
			
			var url = "resources/applications/" + this.appId + "/explorer/";
			if (virtual) {
				url += "v";
			}
			else {
				url += "n";
			}
			
		    this.fileExplorerStore = new zero.ide.data.LazyLoadReadStore(
				{
					url: url, 
					identifierAttr: "path", 
					labelAttr: "name"
				}
			);
			
			this.fileExplorerModel = new zero.ide.widget.appedit.TreeModel (
				{
					"id":"dvfsModelId",
					"jsId":"dvfsModelId2",
					"store": this.fileExplorerStore
				}
			);
		
			var div = document.createElement("div");
			this.fileTreeContainer.domNode.appendChild(div);
			this.fileExplorerWidget = new dijit.Tree (
				{
					"labelAttr": "name",					
					"onClick": dojo.hitch(this,this._fileTreeClicked),					
					"isExpanded": false,
					"model":this.fileExplorerModel,
					//"dndController":"zero.ide.widget.appedit.TreeDndSource", maybe later user DnD when we're ready           		
					"showRoot":false,
             		"checkItemAcceptance":dojo.hitch(this, this.checkItemAcceptance)															
				},
				div
			);
		},
		
		clearViewer : function() {		
			if (this.codeViewerWidget) {				
				dojo.style(this.codeViewerWidget.domNode,"display","none");				
			}
			this._selectedFileItem = null;
		},						
						
		_containsValue: function(	/* items */ items, 
									/* anything */ value){
			//	summary: 
			//		Internal function for search at the items that has the given value.
			//	
			//	items:
			//		The data items to examine.
			//	value:	
			//		The value to match.
			for(var i = 0; i < items.length; ++i){
				if (items[i].path.indexOf(value) !== -1){
					return true; // Boolean
				}
			}
			return false; // Boolean
		},
						
		checkItemAcceptance : function(node, source) {        
            // Get the associated dojo.data item for the target
            var item = dijit.getEnclosingWidget(node).item;
            // Need to check for item because when dropping on a root node,
            // item === null
			// Don't let them drop on to anything but a directory 
            if ((! item) || (!item.directory)) return false;						
			
			// Don't let them drag virtual files
			var srcItem = source.getSelectedItems()[0];
			//if(zlog.isDebug())console.debug("Testing drop on "+item.path + " from "+srcItem.path);			
					
			var isVirtual = (srcItem.dependency && (srcItem.dependency != currentApplication.name));
			if (isVirtual) return false;
								
			// Don't let them drop on to virtual directories
			// BK - the support isn't in the file system service yet to tell us whether
			// a directory is virtual.  It only adds the "dependency" attribute for files			
			// var isVirtual = (item.dependency && (item.dependency != currentApplication.name));
			// if (isVirtual) return false;
			
			// Don't let them drop on to their own parent directory
			var matchStr = srcItem.path.substring(0, srcItem.path.lastIndexOf('/'));			
			if (matchStr == item.path) 
				return false;
											    
			return true;
		},
						
		_enableButtons: function(suspended) {
			if (suspended) {				
				this.resumeButton.attr("disabled", false);
				this.stepOverButton.attr("disabled", false);
				this.stepIntoButton.attr("disabled", false);
				this.stepOutButton.attr("disabled", false);
			} else {
				this.resumeButton.attr("disabled", true);
				this.stepOverButton.attr("disabled", true);
				this.stepIntoButton.attr("disabled", true);
				this.stepOutButton.attr("disabled", true);			
			}
		},
		
		_fileTreeClicked: function(item, node) {
			if(zlog.isDebug())console.debug("_fileTreeClicked:");
			
			// Create a wrapper around the item for use by the dialogs
			this._selectedFileItem = new zero.ide.widget.appedit.FileItem(item, "search");

			if(zlog.isDebug())console.debug(this._selectedFileItem);
			
			if (item.directory) { //item click, expand or collapse this node 				
				if(node.isExpanded){
					node.collapse();
				}else{
			        // Expand the node and get child nodes if not loaded already 
					this.fileExplorerWidget._expandNode(node); 
				}
				return;
			}			
			
			var file = "appbuilder/applications/" + this.appId + "/vfiles" + item.path;
			if(zlog.isDebug())console.debug("DebugView: open file: "+file);
			
			var title = item.path + "  (" + item.dependency + ")";			
			this.loadFileInCodeViewer(file,title);
            
			this.fetchBreakpoints();
			this.codeViewerWidget.clearDebugCursor();
			if(this.lastMatchedSourcePath!=null) {
				if(this.lastMatchedSourcePath.indexOf(item.path) > -1) {
					if(this.stackFrame!=null) {
						this.codeViewerWidget.setDebugCursor(this.stackFrame.lineNumber);
					}					    
				}
			}			
		},
		
		onCodeViewerRefresh: function() {			
			if(this.stackFrame != null) {
				var normalizedPath = this.stackFrame.sourcePath.replace("\\", "/");
				if(this.codeViewerWidget.filePath.indexOf(normalizedPath)!=-1) {
	                // user or debugger opened file where debug cursor is pointing, so scroll to debug cursor
              		this.codeViewerWidget.onGotoLine(this.stackFrame.lineNumber);
				}
			}
		},
				
		loadFileInCodeViewer: function(file,title) {
		    if (this.codeViewerWidget == null) {		    	
				if(zlog.isDebug())console.debug("DebugView: initialize code viewer");
				this.codeViewerWidget = new zero.ide.widget.editor.CodeViewer(
					{
						filePath: file,
						//title: title  // dont show filepath as a tooltip/title
					},
					this.debugView$CodeViewer
				);
				this.codeViewerWidget.startup();
				dojo.connect(this.debugView$CodeViewerPane,"resize",this.codeViewerWidget,"refresh");
				dojo.connect(this.codeViewerWidget.lineNode, 'ondblclick', this, 'onDblClickGutter');
				dojo.connect(this.codeViewerWidget, 'refresh', this, 'onCodeViewerRefresh');
			}
			else {
                if( this.codeViewerWidget.filePath != file ) {
                    this.codeViewerWidget.setFilePath(file);
                    this.codeViewerWidget.clearBreakpoints();                    
                }				
								
				this.codeViewerWidget.clearDebugCursor();
				dojo.style(this.codeViewerWidget.domNode, "display","block");
       			this.codeViewerWidget.onOpen();
			}
                        
            if(!this.isFileDebuggable(file)) {
                this.detachGutterMenu();
            }       
            this.updateExecutableLines();
		},
		
		isFileDebuggable: function(file) {
			var isDebugType = false
	        for(var i = 0; (i < this.debugExts.length) && !isDebugType; i++) {
	            if(file.indexOf(this.debugExts[i]) > 0) {
	                this.attachGutterMenu();
	                isDebugType = true;    
	            }
	        }
			return isDebugType;
		},
				
		_checkboxChanged: function() {
			this._rebuildTree(this.checkbox.checked);
		},
		
  		resume: function()
    	{
    	   this.execDebugAction("resume");
		},
		
		stepOver: function()
    	{
    	   this.execDebugAction("stepOver");    	   
		},
		
		stepInto: function()
    	{
    	   this.execDebugAction("stepInto");
		},
		
		stepOut: function()
    	{
    	   this.execDebugAction("stepOut");
		},
		
		execDebugAction: function(action) 
		{
		   if(this.threadId==null) {
    	       return;
    	   }
    	   
    	   var postD = {
				action: action
    	   };      	   
    	   this.lastAction = action;
    	   
    	   var paramsObj = {  
	        	url: "resources/applications/"+this.appId+"/threads/"+this.threadId,
	        	sync: false,   
            	preventCache: true,
            	headers: {"Accept": "application/json"},
		    	load: dojo.hitch(this, this.updateDebugState(true)),
		    	error: dojo.hitch(this, function(type, error) {        	   
        	      alert("error in debug action: " + action); 
	        	}),	
            	contentType: "application/json",
				mimetype: "application/json",
				postData: dojo.toJson(postD),
            	handleAs: 'json'		
			}            		
			dojo.rawXhrPost(paramsObj);
		},
        
        getRelativeSourcePath: function(absSourcePath) {
            var files = "files/";
            var index = absSourcePath.indexOf(files) + files.length;
            
            // now determine which source dir the file belongs to
            return absSourcePath.substring(index);
        },
        
    
        refreshAll: function() {
            this.updateDebugState(true);
            this.fetchBreakpoints();            
        },
        
        isPhpScript: function(sourcePath) {
            var phpSuffix = ".php";
            sourcePath = sourcePath.toLowerCase();
            return sourcePath.lastIndexOf(phpSuffix) == sourcePath.length - phpSuffix.length;
        }
	}
);
